@@ -783,9 +783,6 @@ seahorse_load_operation_start (SeahorsePGPSource *psrc, const gchar **pattern, g
      
         lop->all = all;
         
-        /* This hashtable contains only allocated strings with no 
-         * values. We allocate the strings in case a key goes away
-         * while we're holding the ids in this table */
         lop->checks = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, NULL);
         
         keys = seahorse_context_get_keys (SCTX_APP (), SEAHORSE_KEY_SOURCE (psrc));
@@ -827,6 +824,8 @@ prepare_import_results (SeahorsePGPOperation *pop, SeahorsePGPSource *psrc)
     SeahorseKeySource *sksrc;
     GList *keys = NULL;
     const gchar **patterns = NULL;
+    GError *err = NULL;
+    gchar *msg;
     GQuark keyid;
     guint i;
     
@@ -844,6 +843,18 @@ prepare_import_results (SeahorsePGPOperation *pop, SeahorsePGPSource *psrc)
             if (GPG_IS_OK (import->result))
                 patterns[i++] = import->fpr;
         }
+        
+        /* See if we've managed to import any ... */
+        if (!patterns[0] && results->considered > 0) {
+            
+            /* ... try and find out why */
+            if (results->no_user_id) {
+                msg = _("Invalid key data (missing UIDs). This may be due to a computer with a date set in the future or a missing self-signature.");
+                g_set_error (&err, SEAHORSE_ERROR, -1, "%s", msg);
+                seahorse_operation_mark_done (SEAHORSE_OPERATION (pop), FALSE, err);
+                return;
+            }
+        }
 
         /* Reload public keys */
         lop = seahorse_load_operation_start (psrc, patterns, LOAD_FULL, 
