@@ -452,6 +452,7 @@ write_gpgme_data (gpgme_data_t data, const gchar *str)
 {
     int len = strlen (str);
     int r = gpgme_data_write (data, str, len);
+
     return len == r;
 }
 
@@ -518,7 +519,8 @@ seahorse_ssh_source_load (SeahorseKeySource *sksrc, SeahorseKeySourceLoad mode,
         
         g_free (ctx.privfile);
         g_free (ctx.pubfile);
-            
+        ctx.privfile = ctx.pubfile = NULL;
+
         if (ctx.matched)
             break;
     }
@@ -540,6 +542,7 @@ seahorse_ssh_source_load (SeahorseKeySource *sksrc, SeahorseKeySourceLoad mode,
         }
         
         g_free (ctx.pubfile);
+		ctx.pubfile = NULL;
     }
     
     /* Load the other keys file */
@@ -556,6 +559,7 @@ seahorse_ssh_source_load (SeahorseKeySource *sksrc, SeahorseKeySourceLoad mode,
         }
         
         g_free (ctx.pubfile);
+		ctx.pubfile = NULL;
     }
 
     /* Clean up and done */
@@ -662,7 +666,7 @@ seahorse_ssh_source_export (SeahorseKeySource *sksrc, GList *keys,
         if (results) {
             
             /* Write the data out */
-            if (write_gpgme_data (data, results)) {
+            if (write_gpgme_data (data, results) == FALSE) {
                 g_set_error (&error, G_FILE_ERROR, g_file_error_from_errno (errno),
                              strerror (errno));
             }
@@ -758,7 +762,7 @@ seahorse_ssh_source_get_property (GObject *object, guint prop_id, GValue *value,
         g_value_set_uint (value, SKEY_SSH);
         break;
     case PROP_KEY_DESC:
-        g_value_set_string (value, _("SSH Key"));
+        g_value_set_string (value, _("Secure Shell Key"));
         break;
     case PROP_LOCATION:
         g_value_set_uint (value, SKEY_LOC_LOCAL);
