@@ -320,7 +320,7 @@ about_cb (BonoboUIComponent *uic, SeahorseApplet *sapplet, const gchar *verbname
                 "artists", artists,
                 "translator-credits", _("translator-credits"),
                 "logo-icon-name", "seahorse-applet",
-                "website", "http://seahorse.sf.net",
+                "website", "http://www.gnome.org/projects/seahorse",
                 "website-label", _("Seahorse Project Homepage"),
                 NULL);
 }
@@ -529,8 +529,8 @@ decrypt_text (const gchar *text, SeahorseApplet *sapplet)
         
         return rawtext;
     } else {
-        seahorse_notification_display(_("Import Failed"),
-                                      _("Keys were found but not imported."),
+        seahorse_notification_display(_("Decrypting Failed"),
+                                      _("Text may be malformed."),
                                       FALSE,
                                       NULL,
                                       GTK_WIDGET(sapplet));
@@ -574,15 +574,17 @@ dvi_received (GtkClipboard *board, const gchar *text, SeahorseApplet *sapplet)
     const gchar *start;         /* Pointer to start of the block */
     const gchar *end;           /* Pointer to end of the block */
     
-    /* No text on clipboard */
+    /* Try to figure out what clipboard contains */
     if (text == NULL)
-        return;
+		type = SEAHORSE_TEXT_TYPE_NONE;
+	else 
+	    type = detect_text_type (text, -1, &start, &end);
     
-    /* Try to figure out what it contains */
-    type = detect_text_type (text, -1, &start, &end);
-    
-    if (type == SEAHORSE_TEXT_TYPE_NONE)
-        return;
+    if (type == SEAHORSE_TEXT_TYPE_NONE || type == SEAHORSE_TEXT_TYPE_PLAIN) {
+        seahorse_notification_display(_("No PGP key or message was found on clipboard"),
+                                      _("No PGP data found."), FALSE, NULL, GTK_WIDGET(sapplet));
+		return;
+	}
     
     switch (type) {
 
